//--------------------------------------------------------------------
//- OpenGL - Partikel Systeme
//- Tutorial 2 - Dynamische Partikel
//--
//- @author Tobias Rick
//- @version 1.0
//- @date: 05.03.02
//--------------------------------------------------------------------

#include "main.h"

#include "partemitter.h"

bool  g_bFullScreen = true;						
HWND  g_hWnd;											
RECT  g_rRect;										
HDC   g_hDC;											
HGLRC g_hRC;											
HINSTANCE g_hInstance;						

GLuint	g_Texture[MAX_TEXTURES];		// Speicher fr Texturen
CTimer g_Timer;
CPartEmitter* g_pEmitter = NULL;
float g_fLastUpdate = 0;

void BuildParticles();

void InitParticles()
{
	// neues Emitterobjekt das bis zu 500 Partikel verwalten kann
	g_pEmitter = new CPartEmitter(500);
	
	g_Timer.Init();

	g_Timer.Update();
	CPartEmitter::m_fCurTime = g_Timer.GetCurrentTime();
	CPartEmitter::m_fFrameTime = g_Timer.GetFrameTime();

	// Partikel zum Emitter hinzufgen
	BuildParticles();
}

void BuildParticles()
{
	for(int i = 0; i < 30 + (rand() % 30) ; i++)
		{
			CParticle* p = new CParticle(g_pEmitter);
			VECTOR vPos = {-10 + ((rand() % 80)-40),
											-10+((rand() % 80)-40),
											-150+((rand() % 20)-10)};
			VECTOR vVel= {( rand()%10)-5*0.001 ,( rand()%10)-5*0.001 ,( rand()%20)-10 *0.0001};
			float clrCol[4] = { 
				((rand() % 100)+1) *0.02,
				((rand() % 100)+1) *0.02,0,1 };
			float s1 = (( rand() % 16)-8)*0.1;
			float s2 = (( rand() % 12)-5)*0.1;
			
			float clrChangeCol[4] = {s1,s2,0};
			float fFade = 0.5f + ((rand() % 5)*0.1);
			float fLife = (rand() % 15 )*0.1 + 4.5 ;
			float fSize = 0.1f + ( (rand() % 10)  -5)*0.1;
			float fChangeSize = ((rand() % 6)-3)*0.1;
			p->Set(vPos,vVel,clrCol,clrChangeCol,fSize,fChangeSize,fLife,fFade);

			g_pEmitter->Add(p);
		}
}



void Init(HWND hWnd)
{
	srand( (unsigned)time( NULL ) );	// Zufallszahlen generieren

	g_hWnd = hWnd;										
	GetClientRect(g_hWnd, &g_rRect);	
	
	InitOpenGL(g_rRect.right, g_rRect.bottom);	

	LoadGLTextures();	// Texturen laden

	InitParticles();	// Partikelsystem initialisieren

	// OpenGL Einstellungen
	glShadeModel(GL_SMOOTH);							
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	
	glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);				
	glClearDepth(1.0f);									
	glDisable(GL_DEPTH_TEST);							
	glClearColor(0.0f,0.0f,0.0f,1.0f);		// Schwarzer Hintergrund
		
	// Blending Einstellungen
	glEnable(GL_BLEND);									
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);		
	
	glEnable(GL_TEXTURE_2D);							

	glBindTexture(GL_TEXTURE_2D,g_Texture[0]);	// Schneetextur auswhlen
}



void KeyPressed()
{
	// keine Benutzereingaben mglich	
}

WPARAM MainLoop()
{
	MSG msg;

	while(1)											
	{													
		if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) 
        { 
			if(msg.message == WM_QUIT)					
				break;
            TranslateMessage(&msg);						
            DispatchMessage(&msg);						
        }
		else											
		{ 
			// Hauptschleife
			g_Timer.Update();
			CPartEmitter::m_fCurTime = g_Timer.GetCurrentTime();
			CPartEmitter::m_fFrameTime = g_Timer.GetFrameTime();

			KeyPressed();
			RenderScene();							
     } 
	}

	// Speicher freigeben
	Shutdown();				
	if (g_pEmitter)
	{
		delete g_pEmitter;
		g_pEmitter = NULL;
	}


	return(msg.wParam);									
}

void DrawBackground()
{
	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glDisable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, g_Texture[1]);

	// Texture-Bewegung
	static float t =0.0f;
	t += g_Timer.GetFrameTime()*0.07;

	glBegin(GL_QUADS);
		glTexCoord2f(0,3-t);
		glVertex3f(-900,-15.5,-900);
		glTexCoord2f(3,3-t); 
		glVertex3f(900,-15.5,-900);
		glTexCoord2f(3,0-t); 
		glVertex3f(300,-15.5,500);
		glTexCoord2f(0,0-t); 
		glVertex3f(-300,-15.5,500);
	glEnd();
	
	glBegin(GL_QUADS);
		glTexCoord2f(0,1-t);
		glVertex3f(-900,-15.5,-900);
		glTexCoord2f(1,1-t); 
		glVertex3f(900,-15.5,-900);
		glTexCoord2f(1,0-t); 
		glVertex3f(300,150.5,500);
		glTexCoord2f(0,0-t); 
		glVertex3f(-300,150.5,500);
	glEnd();
	

	glBindTexture(GL_TEXTURE_2D, g_Texture[0]);
	glEnable(GL_BLEND);
}

void RenderScene() 
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
	glLoadIdentity();									
	
	DrawBackground();	// Hintergrund zeichnen
	
	if(g_Timer.GetCurrentTime() - g_fLastUpdate  > 1.0f)
	{
		BuildParticles();
		g_fLastUpdate = g_Timer.GetCurrentTime();
	}

	g_pEmitter->Update();	// Partikel zeichnen

	SwapBuffers(g_hDC);									
}


LRESULT CALLBACK WinProc(HWND hWnd,UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	LONG    lRet = 0; 
	PAINTSTRUCT    ps;
  
	switch (uMsg)
	{ 
		case WM_SYSCOMMAND:          
			{
				if(wParam== SC_SCREENSAVE)       // Bildschirmschoner deaktivieren
					return false; 
				else
					lRet = DefWindowProc (hWnd, uMsg, wParam, lParam); 
				          
			}			break; 

    case WM_SIZE:										
		if(!g_bFullScreen)								
		{
			ReSize(LOWORD(lParam),HIWORD(lParam));
			GetClientRect(hWnd, &g_rRect);				
		}
        break; 
 
	case WM_PAINT:										
		BeginPaint(hWnd, &ps);					
		EndPaint(hWnd, &ps);						
		break;


	case WM_KEYDOWN:									
		switch (wParam)									
		{
			case VK_ESCAPE:								
				PostQuitMessage(0);					
				break;
		}
		break;

    case WM_CLOSE:										
        PostQuitMessage(0);						
        break; 
     
    default:											
        lRet = DefWindowProc (hWnd, uMsg, wParam, lParam); 
        break; 
    } 
 
    return lRet;									
}
